package techreborn.tiles;

import reborncore.common.IWrenchable;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import reborncore.api.power.EnumPowerTier;
import reborncore.api.recipe.IRecipeCrafterProvider;
import reborncore.api.tile.IInventoryProvider;
import reborncore.common.recipes.RecipeCrafter;
import reborncore.common.tile.TilePowerAcceptor;
import reborncore.common.util.Inventory;
import techreborn.api.Reference;
import techreborn.api.recipe.ITileRecipeHandler;
import techreborn.api.recipe.machines.VacuumFreezerRecipe;
import techreborn.blocks.BlockMachineCasing;
import techreborn.init.ModBlocks;

public class TileVacuumFreezer extends TilePowerAcceptor implements IWrenchable,IInventoryProvider, ITileRecipeHandler<VacuumFreezerRecipe>, IRecipeCrafterProvider {

	public int tickTime;
	public Inventory inventory = new Inventory(3, "TileVacuumFreezer", 64, this);
	public RecipeCrafter crafter;
	public int multiBlockStatus = 0;

	public TileVacuumFreezer() {
		// Input slots
		int[] inputs = new int[1];
		inputs[0] = 0;
		int[] outputs = new int[1];
		outputs[0] = 1;
		crafter = new RecipeCrafter(Reference.vacuumFreezerRecipe, this, 2, 1, inventory, inputs, outputs);
	}

	@Override
	public void func_73660_a() {
		super.func_73660_a();
		crafter.updateEntity();

		if (field_145850_b.func_82737_E() % 20 == 0) {
			multiBlockStatus = checkMachine() ? 1 : 0;
		}
	}

	@Override
	public double getMaxPower() {
		return 64000;
	}

	@Override
	public EnumPowerTier getTier() {
		return EnumPowerTier.MEDIUM;
	}

	@Override
	public boolean wrenchCanSetFacing(EntityPlayer entityPlayer, EnumFacing side) {
		return false;
	}

	@Override
	public EnumFacing getFacing() {
		return getFacingEnum();
	}

	@Override
	public boolean wrenchCanRemove(EntityPlayer entityPlayer) {
		return entityPlayer.func_70093_af();
	}

	@Override
	public float getWrenchDropRate() {
		return 1.0F;
	}

	@Override
	public ItemStack getWrenchDrop(EntityPlayer entityPlayer) {
		return new ItemStack(ModBlocks.AlloySmelter, 1);
	}

	public boolean isComplete() {
		return false;
	}

	@Override
	public void func_145839_a(NBTTagCompound tagCompound) {
		super.func_145839_a(tagCompound);
		crafter.readFromNBT(tagCompound);
	}

	@Override
	public NBTTagCompound func_189515_b(NBTTagCompound tagCompound) {
		super.func_189515_b(tagCompound);
		crafter.writeToNBT(tagCompound);
		return tagCompound;
	}

	public int getProgressScaled(int scale) {
		if (crafter.currentTickTime != 0) {
			return crafter.currentTickTime * scale / crafter.currentNeededTicks;
		}
		return 0;
	}

	public boolean checkMachine() {
		int xDir = EnumFacing.UP.func_82601_c() * 2;
		int yDir = EnumFacing.UP.func_96559_d() * 2;
		int zDir = EnumFacing.UP.func_82599_e() * 2;
		for (int i = -1; i < 2; i++) {
			for (int j = -1; j < 2; j++) {
				for (int k = -1; k < 2; k++) {
					if ((i != 0) || (j != 0) || (k != 0)) {
						if (field_145850_b.func_180495_p(new BlockPos(func_174877_v().func_177958_n() - xDir + i, func_174877_v().func_177956_o() - yDir + j,
								func_174877_v().func_177952_p() - zDir + k)).func_177230_c() != ModBlocks.MachineCasing) {
							return false;
						}
						IBlockState BlockStateContainer = field_145850_b.func_180495_p(new BlockPos(
								func_174877_v().func_177958_n() - xDir + i, func_174877_v().func_177956_o() - yDir + j, func_174877_v().func_177952_p() - zDir + k));
						BlockMachineCasing blockMachineCasing = (BlockMachineCasing) BlockStateContainer.func_177230_c();
						if (blockMachineCasing
								.func_176201_c(BlockStateContainer) != (((i == 0) && (j == 0) && (k != 0))
								|| ((i == 0) && (j != 0) && (k == 0)) || ((i != 0) && (j == 0) && (k == 0)) ? 2
								: 1)) {
							return false;
						}
					} else if (!field_145850_b.func_175623_d(new BlockPos(func_174877_v().func_177958_n() - xDir + i, func_174877_v().func_177956_o() - yDir + j,
							func_174877_v().func_177952_p() - zDir + k))) {
						return false;
					}
				}
			}
		}
		return true;
	}

	@Override
	public boolean canCraft(TileEntity tile, VacuumFreezerRecipe recipe) {
		if (tile instanceof TileVacuumFreezer) {
			if (((TileVacuumFreezer) tile).multiBlockStatus == 1) {
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean onCraft(TileEntity tile, VacuumFreezerRecipe recipe) {
		return true;
	}

	@Override
	public Inventory getInventory() {
		return inventory;
	}

	@Override
	public RecipeCrafter getRecipeCrafter() {
		return crafter;
	}

}
